from pydicom import _storage_sopclass_uids as storage_sopclass_uids
from pydicom.uid import *
from pydicom.uid import PYDICOM_IMPLEMENTATION_UID

DCMAGICK_IMPLEMENTATION_UID = PYDICOM_IMPLEMENTATION_UID
